#include "defs.h"
#include "db.h"
#include "misc.h"
#include "md5/global.h"
#include "md5/md5.h"

/* Generate a key from a password, needed by blowfish. */
int generateKey(BF_KEY *Key, char *Password, int md5)
{
  FILE *Database;
  char HostDB[256];
  int result;
  int i;
  struct dbHosts Entry;

  /* Generate MD5-Hash from Password */
  MD5_CTX context;
  unsigned char digest[16];
  unsigned int len = strlen (Password);
  unsigned char hash[32];

  if (md5 == 1)
  {
    MD5Init(&context);
    MD5Update(&context, Password, len);
    MD5Final(digest, &context);

    for (i = 0; i < 16; i++)
      sprintf (&hash[i*2], "%02x", digest[i]);

    hash[32] = '\0';
    BF_set_key(Key, strlen(hash), hash); /* Generate key */
  } else {
    BF_set_key(Key, strlen(Password), Password); /* Generate key */
  }

  /* open database */
  getHostDB(HostDB, "");
  Database = fopen(HostDB, "r");
  if (Database == NULL) return(ERR_DB_NOT_EXIST);  /* Database doesn't exist yet */

  /* read an entry and check for eof */
  if (fread(&Entry, sizeof(Entry), 1, Database) == 0)
  {
    /* Database is empty */
    fclose(Database);
    return(ERR_DB_NOT_EXIST);
  }

  /* decrypt entry now */
  result = decryptHostEntry(&Entry, Key);
  if (result == ERR_DECRYPTION) {
    /* Maybe an old DB? */
    if (md5 == 1)
    {
      result = generateKey(Key, Password, 0);
    }

    fclose(Database);
    return(result);
  }

  if (result == ERR_OLD_DB) {
    /* sorry, old database */
    fclose(Database);
    if (md5 = 0)
    {
      if (strcmp(Entry.CryptID, "HST") == 0) return(ERR_OLD_OLD_DB);
    } else return(ERR_OLD_DB);
  }

  /* all right */
  fclose(Database);
  return(TRUE);
}

/* Here is a list of all error codes */
char *errorToStr(int error)
{
  switch(error)
  {
    case ERR_NO_ERROR: 			return("No Error");
    case ERR_DB_NOT_EXIST: 		return("Database does not exist yet");
    case ERR_DECRYPTION: 		return("Decryption error (Wrong Password?)");
    case ERR_HOST_EXIST: 		return("Host already exist");
    case ERR_HOST_NOT_EXIST: 		return("Host does not exist");
    case ERR_HOSTNAME_NOT_VALID: 	return("Hostname is not valid");
    case ERR_USERS_EXIST:		return("Cannot delete because users exist");
    case ERR_OLD_DB:			return("v9.2 database. Convert with \"pms_passwd\"");
    case ERR_OLD_OLD_DB:		return("v9.1 database. Convert with \"pms_passwd\"");
    case ERR_MAX_HOSTS_REACHED: 	return("MAX_HOSTS reached");
    case ERR_USER_EXIST: 		return("User already exist");
    case ERR_USER_NOT_EXIST: 		return("User does not exist");
    case ERR_USERNAME_NOT_VALID: 	return("Username is not valid");
    case ERR_MAX_USERS_REACHED: 	return("MAX_USERS for Host reached");
    default: 				return("Unknown error");
  }
}

/* get the name of the host database file depending on the user logged on */
int getHostDB(char *HostDB, char *UserName)
{
  struct passwd *pwdbuf;

  if (strcmp(UserName, "") == FALSE)
  {
    pwdbuf = getpwuid(getuid());
    if (pwdbuf == NULL) return(FALSE);
    strcat(strcpy(HostDB, pwdbuf->pw_dir), "/.pms_db_hosts");
    return(TRUE);
  } else {
    pwdbuf = getpwnam(UserName);
    if (pwdbuf == NULL) return(FALSE);
    strcat(strcpy(HostDB, pwdbuf->pw_dir), "/.pms_db_hosts");
    return(TRUE);
  }
}

/* Exist a host in the database and if not, exist an deleted host, that we can override? */
int existHostEntry(struct dbHosts *Host, BF_KEY *Key)
{
  char HostDB[256];
  struct dbHosts Entry;
  int allowChange = TRUE;
  int result;
  FILE *Database;

  /* Init variables and open database */
  getHostDB(HostDB, "");
  Host->Host = 0;

  Database = fopen(HostDB, "r");
  if (Database == NULL) return(ERR_DB_NOT_EXIST);  /* Database dosn't exist yet */

  /* search the database */
  while(!feof(Database))
  {
    /* read an entry and check for eof */
    if (fread(&Entry, sizeof(Entry), 1, Database) == FALSE) break;

    /* decrypt entry now */
    if (!decryptHostEntry(&Entry, Key)) {
      /* sorry, wrong password */
      fclose(Database);
      return(ERR_DECRYPTION);
    };

    /* is it an deleted entry? */
    if (strlen(Entry.HostName) > 0)
    {
      /* No? is it the entry we search? */
      if (strlen(Entry.HostName) == strlen(Host->HostName))
        if (strcmp(Entry.HostName, Host->HostName) == NTRUE)
        {
          /* Yes, it is! */
          fclose(Database);
          return(ERR_HOST_EXIST); // Host exist
        }
    } else {
      /* Yes, it is an deleted entry, have we found an entry before? */
      if (allowChange)
      {
        /* No, so use this entry as a new place for our new host */
        allowChange = FALSE; /* Do not change the result again */
        Host->Host = Entry.Host; /* use the id from the deleted host */
        result = (ftell(Database) - sizeof(Entry)); /* where is the entry in the database file? */
      }
    }

    /* can we incrase the hostid? */
    if (allowChange)
      if (Host->Host <= Entry.Host)
        Host->Host = Entry.Host + 1;

  }
  fclose(Database);

  if (allowChange) return(ERR_HOST_NOT_EXIST); /* No Host exist */
  return(result); /* No Host exist, but there is an deleted one in the database */
}

/* get a dbHosts structure from the HostID */
struct dbHosts getHostName(int HostID, BF_KEY *Key)
{
  char HostDB[256];
  FILE *Database;
  struct dbHosts Entry;

  /* Init the variables and open the database */
  Entry.Host = ERR_DB_NOT_EXIST; /* a negative hostid means an error, -1 is database not exist (see errorToStr()) */
  getHostDB(HostDB, "");
  Database = fopen(HostDB, "r");
  if (Database == NULL) return(Entry);

  /* search */
  while(!feof(Database))
  {
    /* read and check for eof */
    if (fread(&Entry, sizeof(Entry), 1, Database) == FALSE) break;

    /* decrypt */
    if (!decryptHostEntry(&Entry, Key))
    {
      fclose(Database);
      Entry.Host = ERR_DECRYPTION;  /* error in decryption */
      return(Entry);
    }

    /* is the host not deleted and the hostid our hostid? */
    if (strlen(Entry.HostName) > 0)
     if (Entry.Host == HostID)
     {
       /* yes! */
       fclose(Database);
       return(Entry);
     }
  }

  /* Sorry, but: Host not found */
  strcpy(Entry.HostName, "");
  strcpy(Entry.Comment, "");
  Entry.Host = ERR_HOST_NOT_EXIST;
  fclose(Database);
  return(Entry);
}

/* the opposite of getHostName - get the HostID from the HostName */
int getHostID(char *HostName, BF_KEY *Key)
{
  char HostDB[256];
  FILE *Database;
  struct dbHosts Entry;

  /* init variables and open database */
  getHostDB(HostDB, "");
  Database = fopen(HostDB, "r");
  if (Database == NULL) return(ERR_DB_NOT_EXIST); /* database not exist */

  /* search in database */
  while(!feof(Database))
  {
    /* read and check for eof */
    if (fread(&Entry, sizeof(Entry), 1, Database) == FALSE) break;

    /* decrypt */
    if (!decryptHostEntry(&Entry, Key))
    {
      fclose(Database);
      return(ERR_DECRYPTION); /* sorry, wrong key */
    }


    /* is the host not deleted and the hostname our hostname? */
    if (strlen(Entry.HostName) > 0)
     if (strcmp(Entry.HostName, HostName) == NTRUE)
     {
       /* Yes! */
       fclose(Database);
       return(Entry.Host);
     }
  }

  /* Sorry, host not found */
  fclose(Database);
  return(ERR_HOST_NOT_EXIST);
}

/* get the position of a host in the database */
int findHostEntry(struct dbHosts *Host, BF_KEY *Key)
{
  char HostDB[256];
  FILE *Database;
  struct dbHosts Entry;
  int pos;

  /* Init variables and open database */
  getHostDB(HostDB, "");
  Database = fopen(HostDB, "r");
  if (Database == NULL) return(ERR_HOST_NOT_EXIST); /* Database not exists */

  /* search in database */
  while(!feof(Database))
  {
    /* read an entry and check for eof */
    if (fread(&Entry, sizeof(Entry), 1, Database) == FALSE) break;

    /* decrypt entry */
    if (!decryptHostEntry(&Entry, Key))
    {
      fclose(Database);
      return(ERR_DECRYPTION);       /* wrong key, sorry */
    }

    /* is the hostname not empty, an is it our hostname? */
    if (strlen(Entry.HostName) > 0)
     if (strcmp(Entry.HostName, Host->HostName) == NTRUE)
     {
       /* Yes */
       pos = ftell(Database);
       fclose(Database);
       strcpy(Host->Comment, Entry.Comment); /* set comment */
       Host->Host = Entry.Host;     /* set hostid */
       return(pos - sizeof(Entry)); /* return position */
     }
  }

  /* Host not found */
  fclose(Database);
  return(ERR_HOST_NOT_EXIST);
}

/* Get a list of all hosts in the database */
int getHosts(struct dbHosts Entrys[], BF_KEY *Key)
{
  char HostDB[256];
  FILE *Database;
  struct dbHosts Entry;
  int EntryNo = 0;

  /* Init variables and open database */
  getHostDB(HostDB, "");
  Database = fopen(HostDB, "r");
  if (Database == NULL) return(ERR_DB_NOT_EXIST); /* Database not found */

  /* read the database */
  while(!feof(Database))
  {
    /* read an entry and check for eof */
    if (fread(&Entry, sizeof(Entry), 1, Database) == FALSE) break;

    /* decrypt the entry */
    if (!decryptHostEntry(&Entry, Key))
    {
     fclose(Database);
     return(ERR_DECRYPTION);  /* sorry, wrong key */
    }

    /* Is the entry not deleted? */
    if (strlen(Entry.HostName) > 0)
    {
      /* no, it is'nt. put it in our list */
      if (Entrys != NULL)
      {
        memcpy(&Entrys[EntryNo], &Entry, sizeof(Entry));
        Entrys[EntryNo].HostName[MAX_HOSTNAME_SIZE] = '\0';
        Entrys[EntryNo].Comment[MAX_COMMENT_SIZE] = '\0';
      }
      EntryNo++;
    }
  }

  /* Close database and return number of founded hosts */
  fclose(Database);
  return(EntryNo);
}

/* Add a Host to the Database */
int addHost(char *HostName, char *Comment, BF_KEY *Key)
{
  char HostDB[256];
  FILE *Database;
  struct dbHosts Entry;
  int result;

  /* Is the name valid */
  if (strcmp(HostName, "") == NTRUE) return(ERR_HOSTNAME_NOT_VALID);

  /* Does the host already exist, if not is there an deleted host that we can override? */
  strncpy(Entry.HostName, HostName, MAX_HOSTNAME_SIZE);
  strncpy(Entry.Comment, Comment, MAX_COMMENT_SIZE);
  result = existHostEntry(&Entry, Key);
  if (result == ERR_HOST_NOT_EXIST) result = ERR_DB_NOT_EXIST; /* No host exist, set to -1 to append the host to the database */
  if ((result < 0) && (result != -1)) return(result); /* Error */
  if (getHosts(NULL, Key) == MAX_HOSTS) return(ERR_MAX_HOSTS_REACHED);  /* Maximum reached */

  /* Open Database */
  getHostDB(HostDB, "");
  if (result == ERR_DB_NOT_EXIST)
  {
    Database = fopen(HostDB, "a");
    checkNull(Database, HostDB);
  } else {
    Database = fopen(HostDB, "r+");
    checkNull(Database, HostDB);
    fseek(Database, result, 0);
  }

  /* Encrypt entry, write, close and return */
  encryptHostEntry(&Entry, Key);
  fwrite(&Entry, sizeof(Entry), 1, Database); checkWrite(Database);
  fclose(Database);
  return(Entry.Host); /* All okay */
}

/* Rename a host */
int modifyHost(char *OldName, char *NewName, char *NewComment, BF_KEY *Key, int DeleteRoutine)
{
   char HostDB[256];
   FILE *Database;
   struct dbHosts Entry;
   int result;

   /* Only update comment? */
   if (strcmp(OldName, NewName))
   {
     /* No */
     /* Names valid? */
     if (!DeleteRoutine)
       if (strcmp(NewName, "") == NTRUE) return(ERR_HOSTNAME_NOT_VALID); /* No */
     if (strcmp(OldName, "") == NTRUE) return(ERR_HOSTNAME_NOT_VALID);   /* No */
   }

   /* Search the host */
   strncpy(Entry.HostName, OldName, MAX_HOSTNAME_SIZE);
   result = findHostEntry(&Entry, Key);
   if (result < 0) return(result); /* Error */

   /* Use a new comment? */
   if (NewComment != NULL) strncpy(Entry.Comment, NewComment, MAX_COMMENT_SIZE);

   if (strcmp(OldName, NewName))
   {
     /* Check if the NewName exist */
     strncpy(Entry.HostName, NewName, MAX_HOSTNAME_SIZE);
     if (!DeleteRoutine)
       if (findHostEntry(&Entry, Key) > -1) return(ERR_HOST_EXIST); /* New Host exist */
   }

   /* Open Database */
   getHostDB(HostDB, "");
   Database = fopen(HostDB, "r+");
   checkNull(Database, HostDB);

   /* seek, encrypt, write and close */
   fseek(Database, result, 0);
   encryptHostEntry(&Entry, Key);
   fwrite(&Entry, sizeof(Entry), 1, Database); checkWrite(Database);
   fclose(Database);
   return(ERR_NO_ERROR); /* All okay */
}

/* Delete a Host */
int deleteHost(char *HostName, BF_KEY *Key)
{
  struct dbUsers User;
  int HostID;

  /* Get HostID */
  HostID = getHostID(HostName, Key);
  if (HostID < 0) return(ERR_HOST_NOT_EXIST);

  /* Check if users exist */
  strcpy(User.UserName, "");
  User.Host = HostID;
  if (findUserEntry(&User, Key) == 1) return(ERR_USERS_EXIST);  /* Users exist */

  /* We only need to rename the Host to NULL */
  return(modifyHost(HostName, "", "", Key, TRUE));
}

/* ------ Userdb functions --------- */
/* get the name of the user database file depending on the user logged on */
int getUserDB(char *UserDB, char *UserName)
{
  struct passwd *pwdbuf;

  if (strcmp(UserName, "") == FALSE)
  {
    pwdbuf = getpwuid(getuid());
    if (pwdbuf == NULL) return(FALSE);
    strcat(strcpy(UserDB, pwdbuf->pw_dir), "/.pms_db_users");
    return(TRUE);
  } else {
    pwdbuf = getpwnam(UserName);
    if (pwdbuf == NULL) return(FALSE);
    strcat(strcpy(UserDB, pwdbuf->pw_dir), "/.pms_db_users");
    return(TRUE);
  }
}

/* Exist the given user and if not, is there an deleted entry that we can override? */
int existUserEntry(struct dbUsers *User, BF_KEY *Key)
{
  char UserDB[256];
  FILE *Database;
  struct dbUsers Entry;
  int allowChange = TRUE;
  int result;

  /* Open Database */
  getUserDB(UserDB, "");
  Database = fopen(UserDB, "r");
  if (Database == NULL) return(ERR_DB_NOT_EXIST);  /* Database not exist */

  /* Search */
  while(!feof(Database))
  {
    /* read and decrypt */
    if (fread(&Entry, sizeof(Entry), 1, Database) == 0) break;
    if (!decryptUserEntry(&Entry, Key)) {
     fclose(Database);
     return(ERR_DECRYPTION);  /* Wrong key */
    }

    /* Is the Entry deleted? */
    if (strlen(Entry.UserName) > 0)
    {
      /* No. Is the User the user we search? */
      if (Entry.Host == User->Host)
        if (strlen(Entry.UserName) == strlen(User->UserName))
          if (strcmp(Entry.UserName, User->UserName) == 0)
          {
            /* Yes */
            fclose(Database);
            return(ERR_USER_EXIST);
          }
    } else {
      /* Yes, it's an deleted entry. Have we found an entry before? */
      if (allowChange)
      {
        /* No, so use this entry */
        allowChange = FALSE;
        result = (ftell(Database) - sizeof(Entry));
      }
    }
  }
  fclose(Database);

  if (allowChange) return(ERR_USER_NOT_EXIST); /* User doesn't exist */
  return(result); /* User doesn't exist, but an deleted entry exists */
}


/* get the position of a host in the database */
int findUserEntry(struct dbUsers *User, BF_KEY *Key)
{
  char UserDB[256];
  FILE *Database;
  struct dbUsers Entry;
  int pos;

  /* Open Database */
  getUserDB(UserDB, "");
  Database = fopen(UserDB, "r");
  if (Database == NULL) return(ERR_DB_NOT_EXIST);

  /* search in database */
  while(!feof(Database))
  {
    /* read an entry and check for eof */
    if (fread(&Entry, sizeof(Entry), 1, Database) == FALSE) break;

    /* decrypt entry */
    if (!decryptUserEntry(&Entry, Key))
    {
     fclose(Database);
     return(ERR_DECRYPTION);       /* Sorry, wrong key */
    }

    /* Is the given UserName Empty? */
    if (strcmp(User->UserName, "") == NTRUE)
    {
      /* Yes, compare with HostID */
      if (Entry.Host == User->Host)
        if (strlen(Entry.UserName) > 0)
        {
          /* Found */
          fclose(Database);
          return(1);
        }
    } else {
      /* No, compare with HostName */
      if (strlen(Entry.UserName) > 0)
        if (strcmp(Entry.UserName, User->UserName) == 0)
          if (Entry.Host == User->Host)
            {
              /* Found */
              pos = ftell(Database);
              fclose(Database);
              strncpy(User->Password, Entry.Password, MAX_PASSWORD_SIZE);
              strncpy(User->Comment, Entry.Comment, MAX_COMMENT_SIZE);
              return(pos - sizeof(Entry));
            }
    }
  }

  /* User not found */
  fclose(Database);
  return(ERR_USER_NOT_EXIST);
}


/* Get a list of all user with HostID in the database */
int getUsers(struct dbUsers Entrys[], int HostID, BF_KEY *Key)
{
  char UserDB[256];
  FILE *Database;
  struct dbUsers Entry;
  int EntryNo = 0;

  /* Init variables and open database */
  getUserDB(UserDB, "");
  Database = fopen(UserDB, "r");
  if (Database == NULL) return(ERR_DB_NOT_EXIST);

  /* read the database */
  while(!feof(Database))
  {
    /* read an entry and check for eof */
    if (fread(&Entry, sizeof(Entry), 1, Database) == 0) break;

    /* decrypt the entry */
    if (!decryptUserEntry(&Entry, Key))
    {
     fclose(Database);
     return(ERR_DECRYPTION);  /* sorry, wrong key */
    }

    if (Entry.Host == HostID || HostID == -1)
    {
      /* Is the entry not deleted? */
      if (strlen(Entry.UserName) > 0)
      {
        /* no, it is'nt. put it in our list */
        if (Entrys != NULL)
        {
          memcpy(&Entrys[EntryNo], &Entry, sizeof(Entry));
          Entrys[EntryNo].UserName[MAX_USERNAME_SIZE] = '\0';
          Entrys[EntryNo].Password[MAX_PASSWORD_SIZE] = '\0';
          Entrys[EntryNo].Comment[MAX_COMMENT_SIZE] = '\0';
        }
        EntryNo++;
      }
    }
  }

  /* Close database and return number of founded users */
  fclose(Database);
  return(EntryNo);
}

/* Add an User to the Database */
int addUser(char *UserName, char *Password, char *Comment, int HostID, BF_KEY *Key)
{
  char UserDB[256];
  FILE *Database;
  struct dbUsers Entry;
  struct dbHosts HostEntry;
  int result;

  /* Check Username */
  if (strcmp(UserName, "") == 0) return(ERR_USERNAME_NOT_VALID); /* Illigal name */

  /* Exist HostID? */
  HostEntry = getHostName(HostID, Key);
  if (HostEntry.Host == ERR_DECRYPTION) return(ERR_DECRYPTION); /* Wrong password */
  if (HostEntry.Host == ERR_HOST_NOT_EXIST) return(ERR_HOST_NOT_EXIST); /* Host doesn't exist */

  /* Copy UserName, Password, etc. into a structure */
  strncpy(Entry.UserName, UserName, MAX_USERNAME_SIZE);
  strncpy(Entry.Password, Password, MAX_PASSWORD_SIZE);
  strncpy(Entry.Comment, Comment, MAX_COMMENT_SIZE);
  Entry.Host = HostID;

  /* Do user already exist? */
  result = existUserEntry(&Entry, Key);
  if (result == ERR_DECRYPTION) return(ERR_DECRYPTION);   /* Wrong password */
  if (result == ERR_USER_EXIST) return(ERR_USER_EXIST); /* User exists */

  /* Is MAX_USERS reached? */
  if (getUsers(NULL, HostID, Key) == MAX_USERS) return(ERR_MAX_USERS_REACHED);

  /* Open Database... */
  getUserDB(UserDB, "");
  if (result == ERR_DB_NOT_EXIST || result == ERR_USER_NOT_EXIST)
  {
    Database = fopen(UserDB, "a");
    checkNull(Database, UserDB);
  } else {
    Database = fopen(UserDB, "r+");
    checkNull(Database, UserDB);
    fseek(Database, result, 0);
  }
  /* ...encrypt entry... */
  encryptUserEntry(&Entry, Key);
  /* ...write entry...*/
  fwrite(&Entry, sizeof(Entry), 1, Database); checkWrite(Database);
  /* ...close Database */
  fclose(Database);

  return(ERR_NO_ERROR); /* All right */
}

/* Modify an User */
int modifyUser(char *OldUserName, int OldHostID, char *NewUserName, char *NewPassword, char *NewComment, int NewHostID, BF_KEY *Key, int DeleteRoutine)
{
   char UserDB[256];
   FILE *Database;
   struct dbUsers Entry;
   struct dbUsers NewEntry;
   int result, result2;

   /* Check Username */
   if (strcmp(OldUserName, "") == 0) return(ERR_USERNAME_NOT_VALID); /* Illigal name */

   /* Copy OldUsername, OldHostID into a structure */
   strncpy(Entry.UserName, OldUserName, MAX_USERNAME_SIZE);
   Entry.Host = OldHostID;
   /* Find old user */
   result = findUserEntry(&Entry, Key);
   if (result < 0) return(result); /* Error! */

   /* Check NewUserName, etc */
   if (NewUserName == NULL) strncpy(NewEntry.UserName, Entry.UserName, MAX_USERNAME_SIZE); else strncpy(NewEntry.UserName, NewUserName, MAX_USERNAME_SIZE);
   if (NewPassword == NULL) strncpy(NewEntry.Password, Entry.Password, MAX_PASSWORD_SIZE); else strncpy(NewEntry.Password, NewPassword, MAX_PASSWORD_SIZE);
   if (NewComment == NULL) strncpy(NewEntry.Comment, Entry.Comment, MAX_COMMENT_SIZE); else strncpy(NewEntry.Comment, NewComment, MAX_COMMENT_SIZE);
   if (NewHostID = -1) NewEntry.Host = Entry.Host; else NewEntry.Host = NewHostID;

   /* Is the deleteUser calling? */
   if (!DeleteRoutine)
   {
     /* No, do NewUserName exist? */
     memcpy(&Entry, &NewEntry, sizeof(Entry));
     result2 = findUserEntry(&Entry, Key);
     if (result2 != result)
      if (result2 > ERR_USER_EXIST) return(-11); /* Yes, it does */
   }

   /* Write into Database */
   getUserDB(UserDB, "");
   Database = fopen(UserDB, "r+");
   checkNull(Database, UserDB);
   fseek(Database, result, 0);
   encryptUserEntry(&NewEntry, Key);
   fwrite(&NewEntry, sizeof(NewEntry), 1, Database); checkWrite(Database);
   fclose(Database);
   return(ERR_NO_ERROR); // All okay
}

/* Delete an User */
int deleteUser(char *UserName, int HostID, BF_KEY *Key)
{
  if (UserName == NULL)
  {
    /* Delete all users assigned with HostID */
    struct dbUsers Users[MAX_USERS];
    int count;
    int i;

    count = getUsers(Users, HostID, Key);
    for(i=0; i<count; i++)
    {
      modifyUser(Users[i].UserName, Users[i].Host, "", "", "", -1, Key, TRUE);
    }
    return(ERR_NO_ERROR);
  } else {
    /* Delete only one User */
    return(modifyUser(UserName, HostID, "", "", "", -1, Key, TRUE));
  }
}
