#include "pms_cli.h"

BF_KEY key;

void printHelp()
{
  printf("Help\n");
  printf("----\n");
  printf("Type <cmd> ? for more information about the command\n\n");

  printf("Host functions:\n");
  printf("addHost       - add a host\n");
  printf("modifyHost    - modify a host\n");
  printf("deleteHost    - delete a host\n");
  printf("enumHosts     - enumerate hosts\n\n");

  printf("User functions:\n");
  printf("addUser       - add an user\n");
  printf("modifyUser    - modify an user\n");
  printf("deleteUser    - delete an user\n");
  printf("enumUsers     - enumerate users\n\n");

  printf("Other functions:\n");
  printf("h             - this screen\n");
  printf("exit          - exit the programm\n\n");
}

void HostAdd(char *param)
{
  if (strcmp(param, "?") == 0)
  {
    printf("addHost <HostName>\n");
    printf("------------------\n");
    printf("Add a Host with name <HostName>\n\n");
    return;
  }

  printf("Adding Host...\n");
  printf("HostName = \"%s\"\n", param);
  printf("Result = %d\n\n", addHost(param, &key));
}

void HostMod(char *param)
{
  char oldHost[MAX_HOSTNAME_SIZE + 1];
  char newHost[MAX_HOSTNAME_SIZE + 1];

  if (strcmp(param, "?") == 0)
  {
    printf("modifyHost <OldHostName> <NewHostName>\n");
    printf("--------------------------------------\n");
    printf("renames <OldHostName> to <NewHostName>\n\n");
    return;
  }

  sscanf(param, "%s %s", oldHost, newHost);

  printf("Modify Host...\n");
  printf("OldHostName = \"%s\"\n", oldHost);
  printf("NewHostName = \"%s\"\n", newHost);
  printf("Result = %d\n\n", modifyHost(oldHost, newHost, &key, 0));
}

void HostDel(char *param)
{
  if (strcmp(param, "?") == 0)
  {
    printf("deleteHost <HostName>\n");
    printf("---------------------\n");
    printf("Delete a Host with name <HostName>\n\n");
    return;
  }

  printf("Deleting Host...\n");
  printf("HostName = \"%s\"\n", param);
  printf("Result = %d\n\n", deleteHost(param, &key));
}

void HostEnum(char *param)
{
  struct dbHosts Hosts[MAX_HOSTS];
  int count, i;

  if (strcmp(param, "?") == 0)
  {
    printf("enumHosts\n");
    printf("---------\n");
    printf("shows a list of all hosts\n\n");
    return;
  }

  printf("Get List...\n");
  count = getHosts(Hosts, &key);
  for(i=0; i<count; i++)
  {
    printf("Host \"%s\" has ID %d\n", Hosts[i].HostName, Hosts[i].Host);
  }
  printf("\nTotal number of Hosts: %d\n\n", count);
}

void UserAdd(char *param)
{
  char UserName[MAX_USERNAME_SIZE + 1];
  char Password[MAX_PASSWORD_SIZE + 1];
  char Comment[MAX_COMMENT_SIZE + 1];
  int HostID;

  if (strcmp(param, "?") == 0)
  {
    printf("AddUser <UserName> <PassWord> <Comment> <HostID>\n");
    printf("------------------------------------------------\n");
    printf("Add an User\n\n");
    return;
  }

  sscanf(param, "%s %s %s %d", &UserName, &Password, &Comment, &HostID);
  printf("Add User...\n");
  printf("UserName: %s\n", UserName);
  printf("PassWord: %s\n", Password);
  printf("Comment: %s\n", Comment);
  printf("HostID: %d\n", HostID);
  printf("Result = %d\n\n", addUser(UserName, Password, Comment, HostID, &key));
}

void UserMod(char *param)
{
  char OldUserName[MAX_USERNAME_SIZE + 1];
  char NewUserName[MAX_USERNAME_SIZE + 1];
  char Password[MAX_PASSWORD_SIZE + 1];
  char Comment[MAX_COMMENT_SIZE + 1];
  int OldHostID, NewHostID;

  if (strcmp(param, "?") == 0)
  {
    printf("modifyUser <OldUserName> <OldHostID> <NewUserName> <NewPassWord> <NewComment> <NewHostID>\n");
    printf("-----------------------------------------------------------------------------------------\n");
    printf("Modify an User\n\n");
    return;
  }

  sscanf(param, "%s %d %s %s %s %d", &OldUserName, &OldHostID, &NewUserName, &Password, &Comment, &NewHostID);
  printf("Modify User...\n");
  printf("OldUserName: %s\n", OldUserName);
  printf("OldHostID: %d\n", OldHostID);
  printf("NewUserName: %s\n", NewUserName);
  printf("NewPassWord: %s\n", Password);
  printf("NewComment: %s\n", Comment);
  printf("NewHostID: %d\n", NewHostID);
  printf("Result = %d\n\n", modifyUser(OldUserName, OldHostID, NewUserName, Password, Comment, NewHostID, &key, 0));
}

void UserDel(char *param)
{
  char UserName[MAX_USERNAME_SIZE + 1];
  int HostID;

  if (strcmp(param, "?") == 0)
  {
    printf("DeleteUser <UserName> <HostID>\n");
    printf("------------------------------\n");
    printf("Delete an User\n\n");
    return;
  }

  sscanf(param, "%s %d", &UserName, &HostID);
  printf("Delete User...\n");
  printf("UserName: %s\n", UserName);
  printf("HostID: %d\n", HostID);
  printf("Result = %d\n\n", deleteUser(UserName, HostID, &key));
}

void UserEnum(char *param)
{
  int HostID;
  struct dbUsers Users[MAX_USERS];
  int count, i;
  char *pEnd;

  if (strcmp(param, "?") == 0)
  {
    printf("EnumUsers <HostID>\n");
    printf("------------------\n");
    printf("Enumerate Users with <HostID>\n\n");
    return;
  }

  HostID = strtol(param, &pEnd, 0);

  printf("Get Users...\n");
  printf("HostID: %d\n\n", HostID);

  count = getUsers(Users, HostID, &key);

  for(i=0; i<count; i++)
  {
    printf("-------------------------------\n");
    printf("User %d\n", i);
    printf("UserName: %s\n", Users[i].UserName);
    printf("Password: %s\n", Users[i].Password);
    printf("Comment : %s\n", Users[i].Comment );
  }
  printf("\nTotal number of Users: %d\n\n", count);
}

void processInput(char *cmd, char *param)
{
  if (strcasecmp(cmd, "addhost") == 0) {HostAdd(param); return;}
  if (strcasecmp(cmd, "modifyhost") == 0) {HostMod(param); return;}
  if (strcasecmp(cmd, "deletehost") == 0) {HostDel(param); return;}
  if (strcasecmp(cmd, "enumhosts") == 0) {HostEnum(param); return;}

  if (strcasecmp(cmd, "adduser") == 0) {UserAdd(param); return;}
  if (strcasecmp(cmd, "deleteuser") == 0) {UserDel(param); return;}
  if (strcasecmp(cmd, "modifyuser") == 0) {UserMod(param); return;}
  if (strcasecmp(cmd, "enumusers") == 0) {UserEnum(param); return;}

  if ((strcasecmp(cmd, "h") == 0) || strcmp(cmd, "help") == 0) { printHelp(); return;}
  if (strcasecmp(cmd, "exit") == 0) exit(0);
  if (strcmp(cmd, "\0") == 0) return;
  printf("Unknown command: \"%s\"\n", cmd);
}


int pms_cli()
{
  char input[100];
  char cmd[50];
  char param[50];
  char *pos;

  printf("Password Management System v0.93 - Very instable Command Line Interface\n");
  printf("Only written for testing resons\n\n");
  printf("Type h for Help\n\n");

  generateKey(&key, "PaSsWoRd", 1);

  while(1)
  {
    printf("pms-v0.9> ");
    gets(input);
    if (strcmp(input, ""))
    {
      sscanf(input, "%s", cmd);
      pos = strstr(input, " ");
      if (pos != NULL)
      {
        strcpy(param, pos + 1);
      } else
        strcpy(param, "");

      processInput(cmd, param);
    }
  }
}
