#include "pms_menu.h"
#include "misc.h"

BF_KEY key;

/* Prompt for a Hostname and add this name to the database */
void addHst(CDKSCREEN *cdkscreen)
{
  char *HostName;
  char *Comment;
  char *mesg[2];
  char temp[100];
  int result;

  /* Ask for a Hostname */
  HostName = InputBox(cdkscreen, CENTER, CENTER, 25, "<C></U>Add a Host", "Hostname: ", "", MAX_HOSTNAME_SIZE);

  /* Had user pressed a stupid key, or pressed enter without any entered */
  if (HostName != NULL && strlen(HostName) > 0)
  {
    /* No? Okay... Let's go on.. */
    Comment = InputBox(cdkscreen, CENTER, CENTER, 25, "<C></U>Add a Host", "Comment: ", "", MAX_COMMENT_SIZE);

    /* All okay? */
    if (Comment != NULL)
    {
      /* Yes! Okay, move on... */
      /* Ask the magic are-you-sure question */
      sprintf(temp, "<C> \"%s\" ", HostName);
      mesg[0] = "<C> Add Host? ";
      mesg[1] = temp;
      if (popupDialog (cdkscreen, mesg, 2, yesno, 2) == 0)
      {
        showProgress(cdkscreen);          /* Show "Progressing..." in a wonderful box ;) */
        result = addHost(HostName, Comment, &key); /* Add the Hostname */
        hideProgress(cdkscreen);          /* Hide "Progressing..." */

        if (result >= ERR_NO_ERROR)
        {
          /* All Okay, thus show it */
          mesg[0] = "<C> Host added successfully ";
          popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
          return;
        } else {
          /* Error, thus show the error message */
          sprintf(temp, "<C> Error: %s ", errorToStr(result));
          mesg[0] = temp;
          popupDialog (cdkscreen, mesg, 1, okaybtn, 1);
          return;
        }
      }
    }
  }

  /* User do not want to add a host */
  mesg[0] = "<C> No Host added ";
  popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
}

/* Show a list of hosts in database and delete users selected */
void delHst(CDKSCREEN *cdkscreen)
{
  struct dbHosts Hosts[MAX_HOSTS];
  int HostCount;
  char *HostName;
  char *mesg[2];
  char temp[100];
  int result;
  int HostID;

  /* Get all hosts */
  HostCount = getHosts(Hosts, &key);

  if(HostCount > 0)
  {
    /* Show a list of Hosts */
    HostName = listHosts(cdkscreen, Hosts, HostCount);
    if (HostName != NULL)
    {
      /* The magic are-you-sure question, already */
      sprintf(temp, "<C> \"%s\" ", HostName);
      mesg[0] = "<C> Delete Host? ";
      mesg[1] = temp;
      if (popupDialog (cdkscreen, mesg, 2, yesno, 2) == 0)
      {
        showProgress(cdkscreen);             /* Show "Progressing..." in a wonderful box ;) */
        HostID = getHostID(HostName, &key);  /* Get HostID from HostName */
        deleteUser(NULL, HostID, &key);      /* to delete all users with HostID xxx */
        result = deleteHost(HostName, &key); /* to delete the Host */
        hideProgress(cdkscreen);             /* Hide "Progressing..." */

        if (result == ERR_NO_ERROR)
        {
          /* All Okay, thus show it */
          mesg[0] = "<C> Host deleted successfully ";
          popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
          return;
        } else {
          /* Error, thus show the error message */
          sprintf(temp, "<C> Error: %s ", errorToStr(result));
          mesg[0] = temp;
          popupDialog (cdkscreen, mesg, 1, okaybtn, 1);
          return;
        }
      }
    }
  } else {
    /* No host exists */
    mesg[0] = "<C> No hosts in database ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
    return;
  }

  /* Nothing done */
  mesg[0] = "<C> No Host deleted ";
  popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
}

/* Show a list of hosts in database and rename selected to a name of users choice */
void renHst(CDKSCREEN *cdkscreen)
{
  struct dbHosts Hosts[MAX_HOSTS];
  int HostCount;
  char *HostName;
  char *NewHostName;
  char *mesg[2];
  char temp[100];
  int result;

  /* Get all hosts */
  HostCount = getHosts(Hosts, &key);

  if(HostCount > 0)
  {
    /* Show a list of Hosts */
    HostName = listHosts(cdkscreen, Hosts, HostCount);
    if (HostName != NULL)
    {
      /* Ask for new hostname */
      sprintf(temp, "<C></U> Rename Host \"%s\" to", HostName);
      NewHostName = InputBox(cdkscreen, CENTER, CENTER, 25, temp, "", "", MAX_HOSTNAME_SIZE);
      if(NewHostName != NULL)
      {
        /* Hey, user: Do you really want this? */
        sprintf(temp, "<C> \"%s\" to \"%s\" ", HostName, NewHostName);
        mesg[0] = "<C> Rename Host? ";
        mesg[1] = temp;
        if (popupDialog (cdkscreen, mesg, 2, yesno, 2) == 0)
        {
          /* Yes, he want */
          showProgress(cdkscreen);                             /* Show "Progressing..." in a wonderful box ;) */
          result = modifyHost(HostName, NewHostName, NULL, &key, 0); /* Perform rename */
          hideProgress(cdkscreen);                             /* Hide "Progressing..." */

          if (result == ERR_NO_ERROR)
          {
            /* All right! */
            mesg[0] = "<C> Host renamed successfully ";
            popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
            return;
          } else {
            /* Oh, no! An error! */
            sprintf(temp, "<C> Error: %s ", errorToStr(result));
            mesg[0] = temp;
            popupDialog (cdkscreen, mesg, 1, okaybtn, 1);
            return;
          }
        }
      }
    }
  } else {
    /* No Hosts there */
    mesg[0] = "<C> No hosts in database ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
    return;
  }

  /* The user do not want to make any changes ... */
  mesg[0] = "<C> No Host modified ";
  popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
}

/* Show a list of hosts in database and change the comment of the selected */
void chgComment(CDKSCREEN *cdkscreen)
{
  struct dbHosts Hosts[MAX_HOSTS];
  int HostCount;
  char *HostName;
  char *NewComment;
  char *mesg[2];
  char temp[100];
  int result;

  /* Get all hosts */
  HostCount = getHosts(Hosts, &key);

  if(HostCount > 0)
  {
    /* Show a list of Hosts */
    HostName = listHosts(cdkscreen, Hosts, HostCount);
    if (HostName != NULL)
    {
      /* Ask for new hostname */
      sprintf(temp, "<C></U> Change Comment of Host \"%s\" to", HostName);
      NewComment = InputBox(cdkscreen, CENTER, CENTER, 50, temp, "", "", MAX_COMMENT_SIZE);
      if(NewComment != NULL)
      {
        /* Hey, user: Do you really want this? */
        mesg[0] = "<C> Change Comment? ";
        if (popupDialog (cdkscreen, mesg, 1, yesno, 2) == 0)
        {
          /* Yes, he want */
          showProgress(cdkscreen);                             /* Show "Progressing..." in a wonderful box ;) */
          result = modifyHost(HostName, HostName, NewComment, &key, 0); /* Perform change */
          hideProgress(cdkscreen);                             /* Hide "Progressing..." */

          if (result == ERR_NO_ERROR)
          {
            /* All right! */
            mesg[0] = "<C> Changed successfully ";
            popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
            return;
          } else {
            /* Oh, no! An error! */
            sprintf(temp, "<C> Error: %s ", errorToStr(result));
            mesg[0] = temp;
            popupDialog (cdkscreen, mesg, 1, okaybtn, 1);
            return;
          }
        }
      }
    }
  } else {
    /* No Hosts there */
    mesg[0] = "<C> No hosts in database ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
    return;
  }

  /* The user do not want to make any changes ... */
  mesg[0] = "<C> No Host modified ";
  popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
}

/* Prompt for User information and add this to the database */
void addUsr(CDKSCREEN *cdkscreen, int HostID)
{
  char UserName[MAX_USERNAME_SIZE + 1];
  char Password[MAX_PASSWORD_SIZE + 1];
  char Comment[MAX_COMMENT_SIZE + 1];
  char *mesg[0];
  char temp[100];
  int result;

  /* Clean variables */
  strcpy(UserName, "");
  strcpy(Password, "");
  strcpy(Comment, "");

  /* Show Usermask and process user input */
  if (UserInfoMask(cdkscreen, UserName, Password, Comment, 0) == TRUE)
  {
    showProgress(cdkscreen);                                      /* Show "Progressing..." */
    result = addUser(UserName, Password, Comment, HostID, &key);  /* Add the user */
    hideProgress(cdkscreen);                                      /* Hide "Progressing..." */
    if (result == ERR_NO_ERROR)
    {
      /* All right! */
      mesg[0] = "<C> User added successfully ";
      popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
      return;
    } else {
      /* Oh, no! An error! */
      sprintf(temp, "<C> Error: %s ", errorToStr(result));
      mesg[0] = temp;
      popupDialog (cdkscreen, mesg, 1, okaybtn, 1);
      return;
    }
  } else {
    /* Usermask aborted */
    mesg[0] = "<C> Aborted ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
  }
}

/* Show a list of users and delete the selected */
void delUsr(CDKSCREEN *cdkscreen, int HostID)
{
  struct dbUsers Users[MAX_USERS];
  int result;
  int userCount;
  char *mesg[2];
  char temp[100];


  /* Get the users out of the database */
  userCount = getUsers(Users, HostID, &key);

  if (userCount <= 0)
  {
    /* No Users there */
    mesg[0] = "<C> No Users in database ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
    return;
  }

  /* List Users */
  result = listUsers(cdkscreen, Users, userCount);
  if (result == -1)
  {
    mesg[0] = "<C> Aborted ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
    return;
  }

  /* The magic are-you-sure question, already */
  sprintf(temp, "<C> \"%s\" ", Users[result].UserName);
  mesg[0] = "<C> Delete User? ";
  mesg[1] = temp;
  if (popupDialog (cdkscreen, mesg, 2, yesno, 2) == 0)
  {
    showProgress(cdkscreen);                                    /* Show "Progressing..." */
    result = deleteUser(Users[result].UserName, HostID, &key);  /* Delete User */
    hideProgress(cdkscreen);                                    /* Hide "Progressing..." */
    if (result == ERR_NO_ERROR)
    {
      /* All right! */
      mesg[0] = "<C> User deleted successfully ";
      popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
      return;
    } else {
      /* Oh, no! An error! */
      sprintf(temp, "<C> Error: %s ", errorToStr(result));
      mesg[0] = temp;
      popupDialog (cdkscreen, mesg, 1, okaybtn, 1);
      return;
    }
  } else {
    /* User says No */
    mesg[0] = "<C> Aborted ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
  }
}

/* Show a list of users and modify selected */
void modUsr(CDKSCREEN *cdkscreen, int HostID)
{
  char UserName[MAX_USERNAME_SIZE + 1];
  char Password[MAX_PASSWORD_SIZE + 1];
  char Comment[MAX_COMMENT_SIZE + 1];
  char *mesg[0];
  char temp[100];
  int result;
  struct dbUsers Users[MAX_USERS];
  int userCount;

  /* Get the users out of the database */
  userCount = getUsers(Users, HostID, &key);
  if (userCount <= 0)
  {
    /* No Users there */
    mesg[0] = "<C> No Users in database ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
    return;
  }

  /* List Users */
  result = listUsers(cdkscreen, Users, userCount);
  if (result == -1)
  {
    mesg[0] = "<C> Aborted ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
    return;
  }

  /* Copy Users structure in variables */
  strcpy(UserName, Users[result].UserName);
  strcpy(Password, Users[result].Password);
  strcpy(Comment, Users[result].Comment);

  /* Show usermask and process user input */
  if (UserInfoMask(cdkscreen, UserName, Password, Comment, 0) == TRUE)
  {
    /* User make all right */
    showProgress(cdkscreen);                                    /* show "Progressing... */
    result = modifyUser(Users[result].UserName, HostID,
                UserName, Password, Comment, HostID, &key, 0);  /* Modify User */
    hideProgress(cdkscreen);                                    /* Hide Progressing... */
    if (result == ERR_NO_ERROR)
    {
      /* All right! */
      mesg[0] = "<C> User modified successfully ";
      popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
      return;
    } else {
      /* Oh, no! An error! */
      sprintf(temp, "<C> Error: %s ", errorToStr(result));
      mesg[0] = temp;
      popupDialog (cdkscreen, mesg, 1, okaybtn, 1);
      return;
    }
  } else {
    /* User aborted */
    mesg[0] = "<C> Aborted ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
  }
}

/* Show a list of users and the information of the selected user */
void shwUsr(CDKSCREEN *cdkscreen, int HostID)
{
  struct dbUsers Users[MAX_USERS];
  int result;
  int userCount;
  char *mesg[2];
  char temp[100];

  /* Get the users out of the database */
  userCount = getUsers(Users, HostID, &key);
  if (userCount <= 0)
  {
    /* No Users there */
    mesg[0] = "<C> No Users in database ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
    return;
  }

  /* List Users */
  result = listUsers(cdkscreen, Users, userCount);
  if (result == -1)
  {
    mesg[0] = "<C> Aborted ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
    return;
  }

  /* Display userinformation */
  UserInfoMask(cdkscreen, Users[result].UserName, Users[result].Password, Users[result].Comment, 1);
}

/* show a list of hosts in database and show the usermenu for this host */
void userMenu(CDKSCREEN *cdkscreen)
{
  char *HostName;
  struct dbHosts Hosts[MAX_HOSTS];
  int HostCount;
  int HostID;
  char *mesg[2];
  char temp[100];
  int selection = 0;

  HostCount = getHosts(Hosts, &key);

  if(HostCount > 0)
  {
    HostName = listHosts(cdkscreen, Hosts, HostCount);
    if (HostName == NULL) return;
    HostID = getHostID(HostName, &key);
    while(selection != 6)
    {
      selection = UserMenu(cdkscreen, HostName);
      switch(selection)
      {
        case 0: /* Show User selected */
          shwUsr(cdkscreen, HostID);
          break;
        case 2: /* Add User selected */
          addUsr(cdkscreen, HostID);
          break;
        case 3: /* Delete User selected */
          delUsr(cdkscreen, HostID);
          break;
        case 4: /* Modify User selected */
          modUsr(cdkscreen, HostID);
          break;
        case 6: /* Exit selected */
          break;
      }
    }
  } else {
    mesg[0] = "<C> No hosts in database ";
    popupDialog(cdkscreen, mesg, 1, okaybtn, 1);
    return;
  }
}

/* main function */
int pms_Menu()
{
  WINDOW *cursesWin;
  CDKSCREEN *cdkscreen;
  CDKLABEL *ProgramName;
  int selection = 0;
  int maxx, maxy;
  char *mesg;
  char temp[100];
  char *pass, *pass2;
  int keyres;

  cursesWin = initscr();
  cdkscreen = initCDKScreen(cursesWin);

  maxx = getmaxx(cdkscreen->window);
  maxy = getmaxy(cdkscreen->window);
  if (maxx < 80 || maxy < 24)
  {
    delwin(cursesWin);
    endCDK();
    printf("Current window (%dx%d) is to small. Need 80x24\n", maxx, maxy);
    exit(1);
  }
  initCDKColor();

  temp[0] = '\0';
  strcat(temp, "<C> Password Management System ");
  strcat(temp, VERSION);
  mesg = temp;
  ProgramName = newCDKLabel(cdkscreen, CENTER, 0, &mesg, 1, FALSE, FALSE);
  drawCDKScreen(cdkscreen);

  /* Ask for Password */
  if (generateKey(&key, "", 1) == -1) // Database exist?
  {
    /* No */
    pass = pwdInputBox(cdkscreen, CENTER, CENTER, 25, "<C> Enter new password ", "", 64);
    if (pass == NULL || pass == "")
    {
      delwin(cursesWin);
      endCDK();
      printf("Aborted.\n");
      exit(1);
    }

    pass2 = pwdInputBox(cdkscreen, CENTER, CENTER, 25, "<C> Retype password ", "", 64);
    if (pass2 == NULL || pass2 == "")
    {
      delwin(cursesWin);
      endCDK();
      printf("Aborted.\n");
      exit(1);
    }

    if (strcmp(pass, pass2) != 0)
    {
      delwin(cursesWin);
      endCDK();
      printf("Passwords does not match.\n");
      exit(1);
    }
  } else {
    /* Yes */
    pass = pwdInputBox(cdkscreen, CENTER, CENTER, 25, "<C> Enter password ", "", 64);
    if (pass == NULL || pass == "")
    {
      delwin(cursesWin);
      endCDK();
      printf("Aborted.\n");
      exit(1);
    }
  }

  keyres = generateKey(&key, pass, 1);
  if (keyres < -1)
  {
    delwin(cursesWin);
    endCDK();
    printf("Error: %s\n", errorToStr(keyres));
    exit(1);
  }

  while(selection != 7)
  {
    selection = MainMenu(cdkscreen);

    switch(selection)
    {
      case 0: /* Add Host selected */
        addHst(cdkscreen);
        break;
      case 1: /* Delete Host selected */
        delHst(cdkscreen);
        break;
      case 2: /* Rename Host selected */
        renHst(cdkscreen);
        break;
      case 3: /* Change comment */
        chgComment(cdkscreen);
        break;
      case 5: /* User menu selected */
        userMenu(cdkscreen);
        break;
      case 7: /* Exit selected */
        break;
    }
  }

  destroyCDKLabel(ProgramName);
  delwin(cursesWin);
  endCDK();
}
