#include "ui.h"

/* Needed for show-/hideProgress */
CDKLABEL *PROGRESS_LABEL = NULL;

/* Needed for listUsers and ProcessListUsers */
char *comments[MAX_HOSTS];

/* This function show the PROGRESS_LABEL */
void showProgress(CDKSCREEN *cdkscreen)
{
  char *mesg;

  if (PROGRESS_LABEL != NULL) return;

  mesg = "<C> Progressing... ";
  PROGRESS_LABEL = newCDKLabel(cdkscreen, CENTER, CENTER, &mesg, 1, TRUE, TRUE);

  refreshCDKScreen(cdkscreen);
}

/* This function hide the PROGRESS_LABEL */
void hideProgress(CDKSCREEN *cdkscreen)
{
  char *mesg;

  if (PROGRESS_LABEL == NULL) return;

  destroyCDKLabel(PROGRESS_LABEL);
  PROGRESS_LABEL = NULL;
  refreshCDKScreen(cdkscreen);
}

/* This function shows a box with an input field */
char *InputBox(CDKSCREEN *cdkscreen, int xpos, int ypos, int width, char *title, char *label, char *preVal, int maxLen)
{
  CDKENTRY *inputBox;
  char *result;
  char *rescpy;

  /* Allocate inputBox */
  inputBox = newCDKEntry(cdkscreen, xpos, ypos, title, label, A_NORMAL, ' ', vMIXED, width, 0, maxLen, TRUE, TRUE);

  /* Show box and process user input */
  result = activateCDKEntry(inputBox, 0);

  if (result != NULL)
  {
    /* if the box was shown */
    /* allocate memory for the return buffer */
    rescpy = malloc(strlen(result) + 1);
    /* copy the result into the buffer */
    strcpy(rescpy, result);
  } else {
    /* if not return NULL */
    rescpy = NULL;
  }

  /* free the inputBox */
  destroyCDKEntry(inputBox);

  /* return the return buffer */
  return(rescpy);
}

/* This function shows a box with an input field, but does not show the user input */
char *pwdInputBox(CDKSCREEN *cdkscreen, int xpos, int ypos, int width, char *title, char *label, int maxLen)
{
  CDKENTRY *inputBox;
  char *result;
  char *rescpy;
  char ret[70];

  /* Allocate inputBox */
  inputBox = newCDKEntry(cdkscreen, xpos, ypos, title, label, A_NORMAL, ' ', vMIXED, width, 0, maxLen, TRUE, TRUE);

  /* Show box and process user input */
  drawCDKEntry(inputBox, ObjOf(inputBox)->box);
  wgetnstr(inputBox->fieldWin, (char *) &ret, 70);
  result = ret;

  if (result != NULL)
  {
    /* if the box was shown */
    /* allocate memory for the return buffer */
    rescpy = malloc(strlen(result) + 1);
    /* copy the result into the buffer */
    strcpy(rescpy, result);
  } else {
    /* if not return NULL */
    rescpy = NULL;
  }

  /* free the inputBox */
  destroyCDKEntry(inputBox);

  /* return the return buffer */
  return(rescpy);
}

/* This function collect hostuser information from user or only shows an user */
int UserInfoMask(CDKSCREEN *cdkscreen, char *Username, char *Password, char *Comment, int ShowOnly)
{
  CDKENTRY *cdkUserName;
  CDKENTRY *cdkPassword;
  CDKENTRY *cdkComment;
  char *entryUser;
  char *entryPass;
  char *entryComment;
  char *mesg[1];
  char *yesnochange[] = {" Yes ", " No ", " Modify again "};
  int result = 2;

  cdkUserName = newCDKEntry(cdkscreen, 7, 5, "Username:", NULL, A_NORMAL, ' ', vMIXED, MAX_USERNAME_SIZE + 2, 0, MAX_USERNAME_SIZE, TRUE, TRUE);
  cdkPassword = newCDKEntry(cdkscreen, 37, 10, "Password:", NULL, A_NORMAL, ' ', vMIXED, MAX_PASSWORD_SIZE + 2, 0, MAX_PASSWORD_SIZE, TRUE, TRUE);
  cdkComment  = newCDKEntry(cdkscreen, 7, 15, "Comment:", NULL, A_NORMAL, ' ', vMIXED, MAX_COMMENT_SIZE + 2, 0, MAX_COMMENT_SIZE, TRUE, TRUE);


  setCDKEntryValue(cdkUserName, Username);
  setCDKEntryValue(cdkPassword, Password);
  setCDKEntryValue(cdkComment, Comment);
  refreshCDKScreen(cdkscreen);

  if (ShowOnly)
  {
    getch();
    result = 0;
  } else {
    while(result == 2)
    {
      entryUser = activateCDKEntry(cdkUserName, 0);
      if (entryUser == NULL || strlen(entryUser) == 0) {result = 1; break;}
      strncpy(Username, entryUser, MAX_USERNAME_SIZE);

      entryPass = activateCDKEntry(cdkPassword, 0);
      if (entryPass == NULL) {result = 1; break;}
      strncpy(Password, entryPass, MAX_PASSWORD_SIZE);

      entryComment = activateCDKEntry(cdkComment, 0);
      if (entryComment == NULL) {result = 1; break;}
      strncpy(Comment, entryComment, MAX_COMMENT_SIZE);

      mesg[0] = "<C> Accept changes? ";
      result = popupDialog(cdkscreen, mesg, 1, yesnochange, 3);
    }
  }
  destroyCDKEntry(cdkUserName);
  destroyCDKEntry(cdkPassword);
  destroyCDKEntry(cdkComment);
  if (result == 0) return(1);
  if (result == 1) return(0);
}

/* This function shows the main menu and return the selected entry index */
int MainMenu(CDKSCREEN *cdkscreen)
{
  int selection = 0;
  CDKSCROLL *mainList;
  char *mainmenu[] = {"Add Host",
                      "Delete Host",
                      "Rename Host",
                      "Change Comment",
                      "--------------",
                      "User functions",
                      "--------------",
                      "Exit"};

  /* Allocate mainList */
  mainList = newCDKScroll(cdkscreen, 5, 5, NONE,
                          11, 16, "<C></U>Main Menu", mainmenu, 8, NONUMBERS,
                          A_REVERSE, TRUE, TRUE);

  mainList->currentItem = 5;
  mainList->currentHigh = 5;

  /* Show mainList and process user input */
  selection = activateCDKScroll(mainList, 0);

  /* Free mainList */
  destroyCDKScroll(mainList);
  return(selection);
}

/* This function shows the user menu and return the selected entry index */
int UserMenu(CDKSCREEN *cdkscreen, char *HostName)
{
  int selection = 0;
  CDKSCROLL *userList;
  char *usermenu[] = {"Show User",
                      NULL,
                      "Add User",
                      "Delete User",
                      "Modify User",
                      NULL,
                      "Return to Main Menu"};
  int seperator_len;
  int i, x;
  char title[100];

  sprintf(title, "<C></U> User Menu for %s ", HostName);

  seperator_len = strlen(title);
  if (seperator_len < 19) seperator_len = 19;

  for(x=0; x<7; x++)
  {
    if (usermenu[x] != NULL) continue;
    usermenu[x] = malloc(strlen(title) + 1);
    for(i=0; i<seperator_len; i++)
      usermenu[x][i] = '-';
    usermenu[x][i] = '\0';
  }

  /* Allocate userList */
  userList = newCDKScroll(cdkscreen, 5, 5, NONE,
                          10, 21, title, usermenu, 7, NONUMBERS,
                          A_REVERSE, TRUE, TRUE);

  /* Show userList and process user input */
  selection = activateCDKScroll(userList, 0);

  /* Free userList */
  destroyCDKScroll(userList);
  return(selection);
}

/* This function shows a list of given hosts and return the selected */
char *listHosts(CDKSCREEN *cdkscreen, struct dbHosts Hosts[], int HostCount)
{
  int selection;
  int i, x;
  CDKSCROLL *hostList;
  CDKENTRY *lblComment;
  char *hostmenu[MAX_HOSTS];

  /* Allocate Memory for hostmenu */
  for(i=0; i<HostCount; i++)
  {
    hostmenu[i] = malloc(strlen(Hosts[i].HostName) + 1);
    strcpy(hostmenu[i], Hosts[i].HostName);
  }

  /* Sort hostmenu */
  sortList(hostmenu, HostCount);

  /* Allocate hostList */
  hostList = newCDKScroll(cdkscreen, CENTER, 5, RIGHT,
                          10, 50, "<C></U>Select Host", hostmenu, HostCount, NONUMBERS,
                          A_REVERSE, TRUE, TRUE);

  for(i=0; i<HostCount; i++)
   for(x=0; x<HostCount; x++)
   {
     if(strcmp(hostmenu[i], Hosts[x].HostName) == 0)
     {
       comments[i] = Hosts[x].Comment;
     }
   }

  lblComment = newCDKEntry(cdkscreen, 3, 17, NULL, "Comment: ", A_NORMAL, ' ', vMIXED, MAX_COMMENT_SIZE + 2, 0, MAX_COMMENT_SIZE, FALSE, FALSE);
  setCDKEntry(lblComment, comments[0], 0, MAX_COMMENT_SIZE, TRUE);
  drawCDKEntry(lblComment, 0);

  setCDKScrollPostProcess(hostList, (PROCESSFN)ProcessList, lblComment);

  /* Show list and process user input */
  selection = activateCDKScroll(hostList, 0);

  /* Free hostList */
  destroyCDKScroll(hostList);
  destroyCDKEntry(lblComment);

  /* Free Memory */
  for(i=0; i<HostCount; i++)
   if (i != selection) free(hostmenu[i]); /* Free all, but the selected */

  if (selection > -1)
   /* selection valid */
   return(hostmenu[selection]);
  else
   /* invalid selection */
   return(NULL);
}

/* This function shows a list of given users and return the selected */
int listUsers(CDKSCREEN *cdkscreen, struct dbUsers Users[], int UserCount)
{
  int selection;
  int i, x;
  CDKSCROLL *userList;
  CDKENTRY *lblComment;
  char *usermenu[MAX_USERS];

  /* Allocate Memory for usermenu */
  for(i=0; i<UserCount; i++)
  {
    usermenu[i] = malloc(strlen(Users[i].UserName) + 1);
    strcpy(usermenu[i], Users[i].UserName);
  }

  /* Sort usermenu */
  sortList(usermenu, UserCount);

  /* Allocate userList */
  userList = newCDKScroll(cdkscreen, CENTER, 5, RIGHT,
                          10, 50, "<C></U>Select User", usermenu, UserCount, NONUMBERS,
                          A_REVERSE, TRUE, TRUE);


  for(i=0; i<UserCount; i++)
   for(x=0; x<UserCount; x++)
   {
     if(strcmp(usermenu[i], Users[x].UserName) == 0)
     {
       comments[i] = Users[x].Comment;
     }
   }

  lblComment = newCDKEntry(cdkscreen, 3, 17, NULL, "Comment: ", A_NORMAL, ' ', vMIXED, MAX_COMMENT_SIZE + 2, 0, MAX_COMMENT_SIZE, FALSE, FALSE);
  setCDKEntry(lblComment, comments[0], 0, MAX_COMMENT_SIZE, TRUE);
  drawCDKEntry(lblComment, 0);

  setCDKScrollPostProcess(userList, (PROCESSFN)ProcessList, lblComment);

  /* Show list and process user input */
  drawCDKLabel(lblComment, 0);
  selection = activateCDKScroll(userList, 0);

  /* Free userList */
  destroyCDKScroll(userList);
  destroyCDKEntry(lblComment);

  if(selection != -1)   /* Invalid selection? */
  {
    /* No, search the entry selected */
    for(i=0; i<UserCount; i++)
     if(strcmp(Users[i].UserName, usermenu[selection]) == 0)
     {
       selection = i;
       break;
     }
  }
  /* Free Memory */
  for(i=0; i<UserCount; i++)
   free(usermenu[i]); /* Free all */

  return(selection);
}

int ProcessList(EObjectType cdktype, CDKSCROLL *object, CDKENTRY *clientData, chtype input)
{
  setCDKEntry(clientData, comments[object->currentItem], 0, MAX_COMMENT_SIZE, TRUE);
  drawCDKEntry(clientData, 0);
}
